#!/usr/bin/env python
# -*- coding: utf-8 -*-
from PySide2.QtWebEngineWidgets import QWebEngineView, QWebEnginePage
from PySide2.QtCore import QUrl
from PySide2.QtWebEngineCore import QWebEngineUrlRequestInterceptor
import webbrowser
import PySide2


class WebEnginePage(QWebEnginePage):

    def javaScriptConsoleMessage(self, level, message, lineNumber,
                                 sourceID):  # real signature unknown; restored from __doc__
        """ javaScriptConsoleMessage(self, level: PySide2.QtWebEngineWidgets.QWebEnginePage.JavaScriptConsoleMessageLevel, message: str, lineNumber: int, sourceID: str) """
        print("javaScriptConsoleMessageCall#message = %s" % (message))

    def acceptNavigationRequest(self, url, _type, isMainFrame):
        if _type == QWebEnginePage.NavigationTypeLinkClicked:
            webbrowser.open(url.url(), new=1, autoraise=True)
            return False
        return True

    # 处理https错误提示
    def certificateError(self, certificateError: PySide2.QtWebEngineWidgets.QWebEngineCertificateError) -> bool:
        return True


class WebEngineUrlRequestInterceptor(QWebEngineUrlRequestInterceptor):
    def __init__(self, parent=None):
        QWebEngineUrlRequestInterceptor.__init__(self, parent)

    def interceptRequest(self, info: PySide2.QtWebEngineCore.QWebEngineUrlRequestInfo):
        print(info.requestUrl().url())
        return False


class PreView(QWebEngineView):

    def __init__(self, *args, **kwargs):
        super(PreView, self).__init__(*args, **kwargs)
        # self.setSizePolicy(QSizePolicy.MinimumExpanding, QSizePolicy.Expanding)
        self.init()
        self.setPage(WebEnginePage(self))
        self.page().profile().setUrlRequestInterceptor(self.requestInterceptor)
        self.page().profile().setUseForGlobalCertificateVerification(False)
        self.loadFinishStatus = False
        self.needCall = list()
        self.preContent = None
        self.loadFinished.connect(self.loadFinishCall)

        self.percentageOfTop = 0

    def init(self):
        self.requestInterceptor = WebEngineUrlRequestInterceptor(self)
        self.leftHtml = '''<!DOCTYPE html>
        <html lang="zh-cn">
        <head>
            <meta charset="UTF-8"/>
            <meta content="width=device-width, initial-scale=1.0" name="viewport"/>
            <meta content="IE=edge, chrome=1" http-equiv="X-UA-Compatible"/>
            <meta content="text/html;charset=utf-8" http-equiv="content-type"/>
            <link href="https://www.catbro.cn/css/matery.css" rel="stylesheet">
            <link href="https://cdn.bootcss.com/github-markdown-css/3.0.1/github-markdown.min.css" rel="stylesheet">
            <link rel="stylesheet"
                  href="https://cdn.jsdelivr.net/gh/highlightjs/cdn-release@9.16.2/build/styles/default.min.css">
            <script type="text/javascript"
                    src="https://cdn.jsdelivr.net/gh/highlightjs/cdn-release@9.16.2/build/highlight.min.js"></script>
            <script type="text/javascript" src="https://cdnjs.cloudflare.com/ajax/libs/mermaid/8.4.4/mermaid.min.js"></script>
            <script type="text/javascript"
                    src="https://cdnjs.cloudflare.com/ajax/libs/mathjax/2.7.5/MathJax.js?config=TeX-MML-AM_CHTML"></script>
            <script type="text/javascript">
                MathJax.Hub.Config({
                    tex2jax: {
                        inlineMath: [
                            ['$', '$'],
                            ['(', ')']
                        ]
                    }
                });
            </script>
            <style>
                img {
                    max-width: 100%;
                }
                pre {
                    padding: 1.5rem !important;
                    margin: 1rem 0 !important;
                    background: #272822;
                    overflow: auto;
                    border-radius: 0.35rem;
                    tab-size: 4;
                    display: block;
                    font-size: 87.5%;
                    color: #212529;
                }
                pre code {
                    border-radius: 2px;
                    font-family: Inconsolata, Monaco, Consolas, 'Courier New', Courier, monospace;
                    word-break: normal;
                    font-size: inherit;
                    padding: 0;
                    color: #e8eaf6;
                    background-color: #272822;
                }
                body {
                    margin: 8px !important;
                }
            </style>
        </head>
        <body id="preArea">
        <div id="preMkContent">'''

        self.rightHtml = '''
        </div>
          <script>
              var config = {
                theme:'default',
                logLevel:'fatal',
                securityLevel:'strict',
                startOnLoad:true,
                arrowMarkerAbsolute:false,
            
                flowchart:{
                  htmlLabels:true,
                  curve:'linear',
                },
                sequence:{
                  diagramMarginX:50,
                  diagramMarginY:10,
                  actorMargin:50,
                  width:150,
                  height:65,
                  boxMargin:10,
                  boxTextMargin:5,
                  noteMargin:10,
                  messageMargin:35,
                  mirrorActors:true,
                  bottomMarginAdj:1,
                  useMaxWidth:true,
                  rightAngles:false,
                  showSequenceNumbers:false,
                },
                gantt:{
                  titleTopMargin:25,
                  barHeight:20,
                  barGap:4,
                  topPadding:50,
                  leftPadding:75,
                  gridLineStartPadding:35,
                  fontSize:11,
                  fontFamily:'"Open-Sans", "sans-serif"',
                  numberSectionStyles:4,
                  axisFormat:'%Y-%m-%d',
                }
              };
              mermaid.initialize(config);
          window.reloadMermaid = function reloadMermaid() {
            mermaid.init({
                noteMargin: 10
            }, ".mermaid");
          
          }
          </script>

        </body>
        </html>
        '''

    def urlChanged(self, *args, **kwargs):  # real signature unknown
        print("urlChanged# args:%s and kwargs:%s" % (args, kwargs))
        QWebEngineView.urlChanged(self, *args, **kwargs)

    def loadStarted(self, *args, **kwargs):  # real signature unknown
        print("loadStarted# args:%s and kwargs:%s" % (args, kwargs))
        QWebEngineView.loadStarted(self, *args, **kwargs)

    def loadFinishCall(self, *args, **kwargs):  # real signature unknown
        print("loadFinishCall# args:%s and kwargs:%s" % (args, kwargs))
        self.loadFinishStatus = True
        for call in self.needCall:
            call()
        self.needCall.clear()

    def setPreContent(self, data, force: bool = False):
        if force:
            self.loadFinishStatus = False
            self.preContent = data
            html = self.leftHtml + data + self.rightHtml
            self.page().setHtml(html)
            self.scrollContent(self.percentageOfTop)
        elif not self.preContent:
            self.preContent = data
            html = self.leftHtml + data + self.rightHtml
            self.page().setHtml(html)
        else:
            self.preContent = data
            if self.loadFinishStatus:
                self.realLoadPreContent()
            else:
                self.needCall.append(self.realLoadPreContent)

    def realLoadPreContent(self):
        if self.loadFinishStatus:
            # self.preContent = str(self.preContent).replace("'", "\\\'")
            # self.preContent = str(self.preContent).replace('"', "\\\"")

            javascriptStr = 'window.document.getElementById("preMkContent").innerHTML = `%s`' % str(
                self.preContent)
            self.page().runJavaScript(javascriptStr)

    def scrollContent(self, percentageOfTop=None):
        self.percentageOfTop = percentageOfTop

        if self.loadFinishStatus:
            self.realScrollContent()
        else:
            self.needCall.append(self.realScrollContent)

    def realScrollContent(self):
        if self.loadFinishStatus:
            javascriptStr = '''var scrollingElement = (document.scrollingElement || document.body);
scrollingElement.scrollTop = scrollingElement.scrollHeight * %s;''' % self.percentageOfTop
        self.page().runJavaScript(javascriptStr)
        self.page().runJavaScript('window.reloadMermaid()')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from PySide2.QtWidgets import QDialog, QVBoxLayout, QHBoxLayout, QComboBox, QPushButton


class ComboBoxDialog(QDialog):
    def __init__(self, prent=None):
        QDialog.__init__(self, prent)
        self.init()
        self.initUI()
        self.initEvent()

    def init(self):
        self.mainLayout = QVBoxLayout()
        self.comboBox = QComboBox(self)
        self.bottomLayout = QHBoxLayout()

        self.cancel = QPushButton("cancel")
        self.ok = QPushButton('ok')
        self.result = ""

    def initUI(self):
        self.mainLayout.addWidget(self.comboBox)
        self.bottomLayout.addWidget(self.cancel)
        self.bottomLayout.addWidget(self.ok)
        self.mainLayout.addLayout(self.bottomLayout)
        self.setLayout(self.mainLayout)

    def initEvent(self):
        self.cancel.clicked.connect(self.cancelCall)
        self.ok.clicked.connect(self.okCall)

    def initData(self, selectList: list = None):
        for item in selectList:
            self.comboBox.addItem(item)

    def cancelCall(self):
        self.reject()

    def okCall(self):
        self.result = self.comboBox.currentText()
        self.accept()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from PySide2.QtWidgets import QWidget, QVBoxLayout, QTabWidget, QHBoxLayout, QToolBar, QMessageBox
from PySide2.QtCore import QMargins, QFileInfo, Signal
from PySide2.QtGui import QTextDocument
from .DataClass import FileInfo
from .MkEdit import MkEdit
from menusbar.core import MemusBar
import os


class EditCenter(QWidget):
    createEditSignal = Signal(MkEdit)
    destoryEditSignal = Signal(MkEdit)
    # Tab change signal
    currentTabChangedSignal = Signal(int)

    currentEditPageChangedSignal = Signal(int)

    def __init__(self, parent=None):
        QWidget.__init__(self, parent)
        self.init()

    def startInit(self):
        self._initUI()
        self._initData()
        self._initEvent()

    def __del__(self):
        pass

    def init(self):
        self.toolbar: QToolBar = None

    def _initEvent(self):
        self.getMenusBar.openFileSignal.connect(self.openFileCallBack)
        self.getMenusBar.handlerBeforeEvent = self.handlerBeforeEvent
        self.getMenusBar.handlerAfterEvent = self.handlerAfterEvent

    def handlerBeforeEvent(self, type: int, filePath: str) -> bool:

        if type == self.getMenusBar.HANLDER_DELETE_FILE:
            msgBox = QMessageBox()
            msgBox.setFixedWidth(200)
            msgBox.setFixedHeight(200)
            msgBox.setText("删除提示")
            msgBox.setInformativeText("删除后将无法恢复，确认要删除：%s 文件？" % os.path.basename(filePath))
            msgBox.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
            msgBox.setDefaultButton(QMessageBox.Yes)
            ret = msgBox.exec_()
            if ret == QMessageBox.Yes:
                for index in range(0, self.mdiare.count()):
                    tab = self.mdiare.widget(index)
                    if tab.property("title") == filePath:
                        self.mdiare.removeTab(index)
            elif ret == QMessageBox.No:
                return False



        elif type == self.getMenusBar.HANLDER_DELETE_DIR:
            msgBox = QMessageBox()
            msgBox.setFixedWidth(200)
            msgBox.setFixedHeight(200)
            msgBox.setText("删除提示")
            msgBox.setInformativeText("将删除该目录下所有文件，确认要删除：%s 目录？" % os.path.basename(filePath))
            msgBox.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
            msgBox.setDefaultButton(QMessageBox.Yes)
            ret = msgBox.exec_()
            if ret == QMessageBox.Yes:
                for index in range(0, self.mdiare.count()):
                    tab = self.mdiare.widget(index)
                    if str(tab.property("title")).startswith(filePath):
                        self.mdiare.removeTab(index)
            elif ret == QMessageBox.No:
                return False

        return True

    def handlerAfterEvent(self, type: int, filePath: str):
        pass

    def openFileCallBack(self, file: QFileInfo):
        if file.isFile():
            fileInfo = FileInfo(file.fileName(), file.filePath())
            self.openFileInfo(fileInfo)

    def openFileInfo(self, value: FileInfo):
        has = False
        for index in range(0, self.mdiare.count()):
            tab = self.mdiare.widget(index)
            if tab and tab.property("title") == value.filePath:
                self.mdiare.setCurrentIndex(index)
                has = True
                break
        if not has:
            edit = MkEdit(self)
            edit.loadData(value.filePath)
            edit.setProperty("title", value.filePath)
            self.mdiare.addTab(edit, value.fileName)
            self.mdiare.setTabToolTip(self.mdiare.count() - 1, value.filePath)
            self.mdiare.setCurrentIndex(self.mdiare.count() - 1)
            self.createEditSignal.emit(edit)

    def getCurrentDocment(self) -> QTextDocument:
        edit: MkEdit = self.mdiare.currentWidget()
        return edit.getDocument()

    def getCurrentEdit(self) -> MkEdit:
        edit: MkEdit = self.mdiare.currentWidget()
        return edit

    def tabCloseRequested(self, index):
        _currentTab: MkEdit = self.mdiare.widget(index)
        if not _currentTab.interceptClose():
            self.createEditSignal.emit(self.mdiare.widget(index))
            self.mdiare.removeTab(index)

    def tabBarClicked(self, index):
        pass
    def tabBarDoubleClicked(self, index):
        pass

    def currentTabChanged(self, index):
        self.currentTabChangedSignal.emit(index)

    def currentChanged(self, index):
        self.currentEditPageChangedSignal.emit(index)

    def destroy(self, destroyWindow: bool = ..., destroySubWindows: bool = ...):
        QWidget(EditCenter, self).destroy(destroyWindow, destroySubWindows)

    def create(self, arg__1: int = ..., initializeWindow: bool = ..., destroyOldWindow: bool = ...):
        QWidget(EditCenter, self).create(arg__1, initializeWindow, destroyOldWindow)

    def _initUI(self):
        self.mainLayout = QHBoxLayout()
        self.mainLayout.setContentsMargins(QMargins(0, 0, 0, 0))
        self.menuBar = MemusBar(self)
        self.mainLayout.addWidget(self.menuBar)
        self.rightLayout = QVBoxLayout()
        self.rightLayout.setContentsMargins(QMargins(0, 0, 0, 0))

        self.mainTab = QTabWidget(self)
        self.mainTab.setMovable(True)
        self.mainTab.currentChanged.connect(self.currentTabChanged)
        self.mainTab.setTabPosition(QTabWidget.West)
        self.mdiare = QTabWidget(self)
        self.mdiare.setDocumentMode(True)
        # 注册关闭按钮信号
        self.mdiare.tabCloseRequested.connect(self.tabCloseRequested)
        self.mdiare.tabBarClicked.connect(self.tabBarClicked)
        self.mdiare.tabBarDoubleClicked.connect(self.tabBarDoubleClicked)
        self.mdiare.currentChanged.connect(self.currentChanged)
        self.mdiare.setMovable(True)
        self.mdiare.setTabsClosable(True)

        if len(self.extendTabWidget()) <= 0:
            self.rightLayout.addWidget(self.mdiare)
        else:
            self.mainTab.addTab(self.mdiare, "编辑中心")
            for node in self.extendTabWidget():
                name, widget = node
                self.mainTab.addTab(widget, name)

            self.rightLayout.addWidget(self.mainTab)
            self.mainTab.setCurrentIndex(0)

        self.mainLayout.addLayout(self.rightLayout, 1)
        self.setLayout(self.mainLayout)

    def _initData(self):
        pass
        # edit = Edit(self)
        # self.mdiare.addTab(edit, "xxxx")
        # for i in range(10):

    def setToolBar(self, toolBar: QToolBar):
        if toolBar:
            if self.toolbar:
                self.rightLayout.removeWidget(self.toolbar)
            self.toolbar = toolBar
            self.rightLayout.insertWidget(0, self.toolbar)

    def extendTabWidget(self) -> list:
        return list()

    @property
    def getMenusBar(self):
        return self.menuBar
